// COmpile: g++ -o main MMG_test.cpp -I./mmg/install/include -L./mmg/install -lmmg3d -lscotch -O3 -DNDEBUG -frounding-math -std=gnu++17 -march=native

#include "mmg/mmg3d/libmmg3d.h"

#include <cmath>
#include <algorithm>
#include <iostream>
#include <filesystem>

int main(int argc, char *argv[]) {

	MMG5_pMesh	mmgMesh; //
	MMG5_pSol		mmgLs, mmgMet; //
	int np, ne, nt, na; // np = #vertices, ne = #tetra, nt = #triangles, na = #edges
	int ierr;
	char *outputFile;

	// Initialize mesh and sol structures
	mmgMesh = NULL; mmgLs  = NULL; mmgMet = NULL;

	MMG3D_Init_mesh(MMG5_ARG_start,
	                MMG5_ARG_ppMesh, &mmgMesh, 
	                MMG5_ARG_ppLs,   &mmgLs,
	                MMG5_ARG_ppMet,  &mmgMet,
	                MMG5_ARG_end);


	if ( MMG3D_loadMesh(mmgMesh,"mmg.mesh") != 1 )
		exit(EXIT_FAILURE);

  if ( MMG3D_loadSol(mmgMesh,mmgMet,"mmgMet.sol") != 1)
    exit(EXIT_FAILURE);

  if ( MMG3D_loadSol(mmgMesh,mmgLs,"mmgLs.sol") != 1 )
    exit(EXIT_FAILURE);

	// Set global meshing parameters
	if ( MMG3D_Set_iparameter(mmgMesh, mmgLs, MMG3D_IPARAM_iso, 1) != 1 ) exit(EXIT_FAILURE);	// Use level-set
	//if ( MMG3D_Set_dparameter(mmgMesh, mmgLs, MMG3D_DPARAM_ls, 0) != 1 ) exit(EXIT_FAILURE);		// Offset value
	
	if ( MMG3D_Set_iparameter(mmgMesh, mmgLs, MMG3D_IPARAM_nofem, 1) != 1 )  exit(EXIT_FAILURE);	// No finite element mesh???

	//if ( MMG3D_Set_dparameter(mmgMesh, mmgLs, MMG3D_DPARAM_hausd, 0.25*(minFeatureSize+maxFeatureSize)/2) != 1 )  exit(EXIT_FAILURE); 	// Control global Hausdorff distance (on all the boundary surfaces of the mesh) CHECK!!
	if ( MMG3D_Set_dparameter(mmgMesh, mmgLs, MMG3D_DPARAM_hausd, 0.25) != 1 )  exit(EXIT_FAILURE); 	// Control global Hausdorff distance (on all the boundary surfaces of the mesh) CHECK!!
	if ( MMG3D_Set_dparameter(mmgMesh, mmgLs, MMG3D_DPARAM_hgrad, 2.3) != 1 )  exit(EXIT_FAILURE);	// 

//	if ( MMG3D_Set_iparameter(mmgMesh, mmgLs, MMG3D_IPARAM_numsubdomain, 2) != 1 )  exit(EXIT_FAILURE);
//	if ( MMG3D_Set_iparameter(mmgMesh, mmgLs, MMG3D_IPARAM_numsubdomain, 3) != 1 )  exit(EXIT_FAILURE);

	// Dicretize the level-set
	ierr = MMG3D_mmg3dls(mmgMesh, mmgLs, NULL);
	//ierr = MMG3D_mmg3dls(mmgMesh, mmgLs, mmgMet);
	if ( ierr == MMG5_STRONGFAILURE ) {
		fprintf(stdout,"BAD ENDING OF MMG3DLS: UNABLE TO SAVE MESH\n");
		return(ierr);
	} else if ( ierr == MMG5_LOWFAILURE )
		fprintf(stdout,"BAD ENDING OF MMG3DLS\n");
	
	// Save volume mesh
	if ( MMG3D_saveMesh(mmgMesh, "out.mesh") != 1 ) {
		fprintf(stdout, "UNABLE TO SAVE VOLUME MESH\n");
		return(MMG5_STRONGFAILURE);
	}

	// Free the MMG3D5 structures
	MMG3D_Free_all(MMG5_ARG_start,
	               MMG5_ARG_ppMesh, &mmgMesh,
	               MMG5_ARG_ppLs,   &mmgLs,
	               MMG5_ARG_ppMet,  &mmgMet,
	               MMG5_ARG_end);

	// Cleanup
	free(outputFile);
	outputFile = NULL;

	return EXIT_SUCCESS;
}